{*******************************************************}
{                                                       }
{       Advanced SQL statement parser demo              }
{       Visitor for building treeview from parsed       }
{       statement                                       }
{       Copyright (c) 2001 - 2003 AS Gaiasoft           }
{       Created by Gert Kello                           }
{                                                       }
{*******************************************************}

unit gaParserTreeviewBuilder;

interface

uses
  ComCtrls, Classes,
  gaParserVisitor,
  gaAdvancedSQLParser, gaDeleteStm, gaInsertStm, gaSelectStm, gaUpdateStm,
  gaSQLParserHelperClasses, gaSQLFieldRefParsers, gaSQLSelectFieldParsers,
  gaSQLExpressionParsers, gaSQLTableRefParsers;

type
  TgaParserTreeBuilder = class (TgaCustomParserVisitor)
  private
    FParentNodeList: TList;
    FTreeView: TTreeView;
    function GetParentItem: TTreeNode;
  protected
    function AddLevelNode(const NodeText: string): TTreeNode;
    function AddLevelNodeFmt(const NodeFmtText: string; FmtParams: array of 
            const): TTreeNode;
    function AddNode(const NodeText: string): TTreeNode;
    function AddNodeFmt(const NodeFmtText: string; FmtParams: array of const): 
            TTreeNode;
    procedure RemoveLevel;
    property ParentItem: TTreeNode read GetParentItem;
    property TreeView: TTreeView read FTreeView;
  public
    constructor Create(ATreeView: TTreeView);
    destructor Destroy; override;
    procedure Clear;
    procedure VisitCustomSQLStatement(Instance: TgaCustomSQLStatement); 
            override;
    procedure VisitDeleteSQLStatement(Instance: TgaDeleteSQLStatement); 
            override;
    procedure VisitHavingClause(Instance: TgaHavingClause); override;
    procedure VisitInsertSQLStatement(Instance: TgaInsertSQLStatement); 
            override;
    procedure VisitJoinClause(Instance: TgaJoinClause); override;
    procedure VisitJoinClauseList(Instance: TgaJoinClauseList); override;
    procedure VisitJoinOnPredicate(Instance: TgaJoinOnPredicate); override;
    procedure VisitListOfSQLTokenLists(Instance: TgaListOfSQLTokenLists); 
            override;
    procedure VisitNoSQLStatement(Instance: TgaNoSQLStatement); override;
    procedure VisitSecondaryFieldReference(Instance: 
            TgaSecondaryFieldReference); override;
    procedure VisitSelectSQLStatement(Instance: TgaSelectSQLStatement); 
            override;
    procedure VisitSQLDataReference(Instance: TgaSQLDataReference); override;
    procedure VisitSQLExpression(Instance: TgaSQLExpression); override;
    procedure VisitSQLExpressionBase(Instance: TgaSQLExpressionBase); override;
    procedure VisitSQLExpressionConstant(Instance: TgaSQLExpressionConstant); 
            override;
    procedure VisitSQLExpressionFunction(Instance: TgaSQLExpressionFunction); 
            override;
    procedure VisitSQLExpressionList(Instance: TgaSQLExpressionList); override;
    procedure VisitSQLExpressionOperator(Instance: TgaSQLExpressionOperator); 
            override;
    procedure VisitSQLExpressionPart(Instance: TgaSQLExpressionPart); override;
    procedure VisitSQLExpressionPartBuilder(Instance: 
            TgaSQLExpressionPartBuilder); override;
    procedure VisitSQLExpressionSubselect(Instance: TgaSQLExpressionSubselect); 
            override;
    procedure VisitSQLFieldList(Instance: TgaSQLFieldList); override;
    procedure VisitSQLFieldReference(Instance: TgaSQLFieldReference); override;
    procedure VisitSQLFunctionParamsList(Instance: TgaSQLFunctionParamsList); 
            override;
    procedure VisitSQLGroupByList(Instance: TgaSQLGroupByList); override;
    procedure VisitSQLGroupByReference(Instance: TgaSQLGroupByReference); 
            override;
    procedure VisitSQLMultipartExpression(Instance: TgaSQLMultipartExpression); 
            override;
    procedure VisitSQLOrderByList(Instance: TgaSQLOrderByList); override;
    procedure VisitSQLOrderByReference(Instance: TgaSQLOrderByReference); 
            override;
    procedure VisitSQLSelectExpression(Instance: TgaSQLSelectExpression); 
            override;
    procedure VisitSQLSelectField(Instance: TgaSQLSelectField); override;
    procedure VisitSQLSelectFieldList(Instance: TgaSQLSelectFieldList); 
            override;
    procedure VisitSQLSelectReference(Instance: TgaSQLSelectReference); 
            override;
    procedure VisitSQLStatementPart(Instance: TgaSQLStatementPart); override;
    procedure VisitSQLStatementPartList(Instance: TgaSQLStatementPartList); 
            override;
    procedure VisitSQLStoredProcReference(Instance: TgaSQLStoredProcReference); 
            override;
    procedure VisitSQLSubExpression(Instance: TgaSQLSubExpression); override;
    procedure VisitSQLTable(Instance: TgaSQLTable); override;
    procedure VisitSQLTableList(Instance: TgaSQLTableList); override;
    procedure VisitSQLTableReference(Instance: TgaSQLTableReference); override;
    procedure VisitSQLTokenHolderList(Instance: TgaSQLTokenHolderList); 
            override;
    procedure VisitSQLTokenList(Instance: TgaSQLTokenList); override;
    procedure VisitSQLUnrecognizedExpression(Instance: 
            TgaSQLUnrecognizedExpression); override;
    procedure VisitSQLWhereExpression(Instance: TgaSQLWhereExpression); 
            override;
    procedure VisitUnkownSQLStatement(Instance: TgaUnkownSQLStatement); 
            override;
    procedure VisitUpdateSQLStatement(Instance: TgaUpdateSQLStatement); 
            override;
  end;
  

implementation

uses
  SysUtils, gaParserDemoHelpers;

{
***************************** TgaParserTreeBuilder *****************************
}
constructor TgaParserTreeBuilder.Create(ATreeView: TTreeView);
begin
  inherited Create;
  FTreeView := ATreeView;
  FParentNodeList := TList.Create;
end;

destructor TgaParserTreeBuilder.Destroy;
begin
  FParentNodeList.Free;
  inherited;
end;

function TgaParserTreeBuilder.AddLevelNode(const NodeText: string): TTreeNode;
begin
  Result := AddNode(NodeText);
  FParentNodeList.Add(Result);
end;

function TgaParserTreeBuilder.AddLevelNodeFmt(const NodeFmtText: string; 
        FmtParams: array of const): TTreeNode;
begin
  Result := AddLevelNode(Format(NodeFmtText, FmtParams));
end;

function TgaParserTreeBuilder.AddNode(const NodeText: string): TTreeNode;
begin
  Result := TreeView.Items.AddChild(ParentItem, GetReadableString(NodeText));
end;

function TgaParserTreeBuilder.AddNodeFmt(const NodeFmtText: string; FmtParams: 
        array of const): TTreeNode;
begin
  Result := AddNode(Format(NodeFmtText, FmtParams));
end;

procedure TgaParserTreeBuilder.Clear;
begin
  TreeView.Items.Clear;
  FParentNodeList.Clear;
end;

function TgaParserTreeBuilder.GetParentItem: TTreeNode;
begin
  if FParentNodeList.Count > 0 then
    Result := TTreeNode(FParentNodeList.Last)
  else
    Result := nil;
end;

procedure TgaParserTreeBuilder.RemoveLevel;
begin
  FParentNodeList.Delete(FParentNodeList.Count -1);
end;

procedure TgaParserTreeBuilder.VisitCustomSQLStatement(Instance: 
        TgaCustomSQLStatement);
begin
  AddLevelNode(Instance.ClassName + ' as TgaCustomSQLStatement');
  AddLevelNode('All fields');
  Instance.AllFields.AcceptParserVisitor(Self);
  RemoveLevel;
  AddLevelNode('All tables');
  Instance.AllTables.AcceptParserVisitor(Self);
  RemoveLevel;
  RemoveLevel;
end;

procedure TgaParserTreeBuilder.VisitDeleteSQLStatement(Instance: 
        TgaDeleteSQLStatement);
begin
  VisitCustomSQLStatement(Instance);
end;

procedure TgaParserTreeBuilder.VisitHavingClause(Instance: TgaHavingClause);
begin
  AddLevelNode('Having:');
  VisitSQLMultipartExpression(Instance);
  RemoveLevel;
end;

procedure TgaParserTreeBuilder.VisitInsertSQLStatement(Instance: 
        TgaInsertSQLStatement);
begin
  AddLevelNode(Instance.ClassName + ' as TgaInsertSQLStatement');
  
  AddLevelNode('Table to insert');
  Instance.StatementTable.AcceptParserVisitor(Self);
  RemoveLevel;
  if Instance.InsertFields <> nil then
  begin
    AddLevelNode('Insert fields');
    Instance.InsertFields.AcceptParserVisitor(Self);
    RemoveLevel;
  end;
  
  if Instance.InsertValues <> nil then
  begin
    AddLevelNode('Insert values');
    Instance.InsertValues.AcceptParserVisitor(Self);
    RemoveLevel;
  end;
  
  if Instance.SelectStm <> nil then
  begin
    AddLevelNode('Insert select');
    Instance.SelectStm.AcceptParserVisitor(Self);
    RemoveLevel;
  end;
  
  RemoveLevel;
  VisitCustomSQLStatement(Instance);
end;

procedure TgaParserTreeBuilder.VisitJoinClause(Instance: TgaJoinClause);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitJoinClauseList(Instance: TgaJoinClauseList);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitJoinOnPredicate(Instance: 
        TgaJoinOnPredicate);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitListOfSQLTokenLists(Instance: 
        TgaListOfSQLTokenLists);
begin
  Instance.First;
  while not Instance.Eof do
  begin
    Instance.CurrentItem.AcceptParserVisitor(Self);
    Instance.Next;
  end;
end;

procedure TgaParserTreeBuilder.VisitNoSQLStatement(Instance: TgaNoSQLStatement);
begin
  VisitCustomSQLStatement(Instance);
end;

procedure TgaParserTreeBuilder.VisitSecondaryFieldReference(Instance: 
        TgaSecondaryFieldReference);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSelectSQLStatement(Instance: 
        TgaSelectSQLStatement);
begin
  AddLevelNode('Select statement');
  if Instance.WhereClause <> nil then
    Instance.WhereClause.AcceptParserVisitor(Self);
  if Instance.GroupByClause <> nil then
    Instance.GroupByClause.AcceptParserVisitor(Self);
  if Instance.HavingClause <> nil then
    Instance.HavingClause.AcceptParserVisitor(Self);
  if Instance.OrderByClause <> nil then
    Instance.OrderByClause.AcceptParserVisitor(Self);
  RemoveLevel;
  VisitCustomSQLStatement(Instance);
end;

procedure TgaParserTreeBuilder.VisitSQLDataReference(Instance: 
        TgaSQLDataReference);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSQLExpression(Instance: TgaSQLExpression);
begin
  VisitSQLMultipartExpression(Instance);
end;

procedure TgaParserTreeBuilder.VisitSQLExpressionBase(Instance: 
        TgaSQLExpressionBase);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSQLExpressionConstant(Instance: 
        TgaSQLExpressionConstant);
begin
  AddNodeFmt('Constant: "%s"', [Instance.AsString]);
  // TODO : Implement method
end;

procedure TgaParserTreeBuilder.VisitSQLExpressionFunction(Instance: 
        TgaSQLExpressionFunction);
begin
  AddLevelNodeFmt('Function: "%s"', [Instance.FunctionName]);
  AddlevelNode('parameters:');
  Instance.FunctionParameters.First;
  while not Instance.FunctionParameters.Eof do
  begin
    Instance.FunctionParameters.CurrentItem.AcceptParserVisitor(Self);
  
    Instance.FunctionParameters.Next;
  end;
  RemoveLevel;
  RemoveLevel;
end;

procedure TgaParserTreeBuilder.VisitSQLExpressionList(Instance: 
        TgaSQLExpressionList);
begin
  AddLevelNode('Expression list:');
  try
    Instance.First;
    while not Instance.Eof do
    begin
      Instance.CurrentItem.AcceptParserVisitor(Self);
      Instance.Next;
    end;
  finally
    RemoveLevel;
  end;
end;

procedure TgaParserTreeBuilder.VisitSQLExpressionOperator(Instance: 
        TgaSQLExpressionOperator);
begin
  AddNodeFmt('Operator: "%s"', [Instance.AsString]);
end;

procedure TgaParserTreeBuilder.VisitSQLExpressionPart(Instance: 
        TgaSQLExpressionPart);
begin
  Assert(False, 'TgaSQLExpressionPart is abstract class');
end;

procedure TgaParserTreeBuilder.VisitSQLExpressionPartBuilder(Instance: 
        TgaSQLExpressionPartBuilder);
begin
  Assert(False, 'TgaSQLExpressionPartBuilder should never be called for visiting');
end;

procedure TgaParserTreeBuilder.VisitSQLExpressionSubselect(Instance: 
        TgaSQLExpressionSubselect);
begin
  AddLevelNode('Sub-select');
  if Instance.SelectStm <> nil then
    Instance.SelectStm.AcceptParserVisitor(Self);
  RemoveLevel;
end;

procedure TgaParserTreeBuilder.VisitSQLFieldList(Instance: TgaSQLFieldList);
begin
  AddLevelNode('Fields:');
  try
    Instance.First;
    while not Instance.Eof do
    begin
      Instance.CurrentItem.AcceptParserVisitor(Self);
      Instance.Next;
    end;
  finally
    RemoveLevel;
  end;
end;

procedure TgaParserTreeBuilder.VisitSQLFieldReference(Instance: 
        TgaSQLFieldReference);
begin
  AddNodeFmt('Reference to field: "%s"', [Instance.AsString]);
end;

procedure TgaParserTreeBuilder.VisitSQLFunctionParamsList(Instance: 
        TgaSQLFunctionParamsList);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSQLGroupByList(Instance: TgaSQLGroupByList);
begin
  AddLevelNode('Group by:');
  Instance.First;
  while not Instance.Eof do
  begin
    Instance.CurrentItem.AcceptParserVisitor(Self);
    Instance.Next;
  end;
  RemoveLevel;
end;

procedure TgaParserTreeBuilder.VisitSQLGroupByReference(Instance: 
        TgaSQLGroupByReference);
begin
  Instance.Reference.AcceptParserVisitor(Self);
end;

procedure TgaParserTreeBuilder.VisitSQLMultipartExpression(Instance: 
        TgaSQLMultipartExpression);
var
  CurPart: TgaSQLExpressionPart;
begin
  CurPart := Instance.FirstSubPart;
  while CurPart <> nil do
  begin
    CurPart.AcceptParserVisitor(Self);
    CurPart := CurPart.NextExpressionPart;
  end;
end;

procedure TgaParserTreeBuilder.VisitSQLOrderByList(Instance: TgaSQLOrderByList);
begin
  AddLevelNode('Order by:');
  Instance.First;
  while not Instance.Eof do
  begin
    Instance.CurrentItem.AcceptParserVisitor(Self);
    Instance.Next;
  end;
  RemoveLevel;
end;

procedure TgaParserTreeBuilder.VisitSQLOrderByReference(Instance: 
        TgaSQLOrderByReference);
begin
  AddLevelNode('Ordered by:');
  Instance.Reference.AcceptParserVisitor(Self);
  if Instance.OrderedAscending then
    AddNode('ascending')
  else
    AddNode('descending');
  AddNodeFmt('Order descriptor: "%s"', [Instance.OrderDescriptorStr]);
  RemoveLevel;
end;

procedure TgaParserTreeBuilder.VisitSQLSelectExpression(Instance: 
        TgaSQLSelectExpression);
begin
  VisitSQLExpression(Instance);
end;

procedure TgaParserTreeBuilder.VisitSQLSelectField(Instance: TgaSQLSelectField);
begin
  AddLevelNodeFmt('Selected field: "%s"', [Instance.AsString]);
  AddLevelNodeFmt('Select expression: "%s"', [Instance.ValueExpression.AsString]);
  Instance.ValueExpression.AcceptParserVisitor(Self);
  RemoveLevel;
  if Instance.FieldAliasToken <> nil then
    AddNodeFmt('Field alias: "%s"', [Instance.FieldAlias]);
  RemoveLevel;
end;

procedure TgaParserTreeBuilder.VisitSQLSelectFieldList(Instance: 
        TgaSQLSelectFieldList);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSQLSelectReference(Instance: 
        TgaSQLSelectReference);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSQLStatementPart(Instance: 
        TgaSQLStatementPart);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSQLStatementPartList(Instance: 
        TgaSQLStatementPartList);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSQLStoredProcReference(Instance: 
        TgaSQLStoredProcReference);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSQLSubExpression(Instance: 
        TgaSQLSubExpression);
begin
  AddLevelNode('Sub expression');
  VisitSQLMultipartExpression(Instance.ExpressionBody);
  RemoveLevel;
end;

procedure TgaParserTreeBuilder.VisitSQLTable(Instance: TgaSQLTable);
begin
  AddLevelNode(Format('Table: "%s"', [Instance.AsString]));
  try
    AddNodeFmt('Table prefixies: "%s"', [Instance.TablePrefix]);
    AddNodeFmt('Table name: "%s"', [Instance.TableName]);
    AddNodeFmt('Table alias: "%s"', [Instance.TableAlias]);
  finally
    RemoveLevel;
  end;
end;

procedure TgaParserTreeBuilder.VisitSQLTableList(Instance: TgaSQLTableList);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSQLTableReference(Instance: 
        TgaSQLTableReference);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSQLTokenHolderList(Instance: 
        TgaSQLTokenHolderList);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSQLTokenList(Instance: TgaSQLTokenList);
begin
  // TODO : Implement method
  Assert(False, 'Not implemented');
end;

procedure TgaParserTreeBuilder.VisitSQLUnrecognizedExpression(Instance: 
        TgaSQLUnrecognizedExpression);
begin
  AddNodeFmt('Unrecognized: "%s"', [Instance.AsString]);
end;

procedure TgaParserTreeBuilder.VisitSQLWhereExpression(Instance: 
        TgaSQLWhereExpression);
begin
  AddLevelNode('Where expression:');
  VisitSQLMultipartExpression(Instance);
  RemoveLevel;
end;

procedure TgaParserTreeBuilder.VisitUnkownSQLStatement(Instance: 
        TgaUnkownSQLStatement);
begin
  VisitCustomSQLStatement(Instance);
end;

procedure TgaParserTreeBuilder.VisitUpdateSQLStatement(Instance: 
        TgaUpdateSQLStatement);
begin
  VisitCustomSQLStatement(Instance);
end;

end.
