{*******************************************************}
{                                                       }
{       Advanced SQL statement parser demo program      }
{       Copyright (c) 2001 - 2003 AS Gaiasoft           }
{       Created by Gert Kello                           }
{                                                       }
{*******************************************************}

unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, gaAdvancedSQLParser, ActnList, Menus,
  gaParserTreeviewBuilder;

type
  TfrmMain = class(TForm)
    pnlButtons: TPanel;
    pgcSQL: TPageControl;
    tshSQLText: TTabSheet;
    tshNewSQLText: TTabSheet;
    tshTokenInfo: TTabSheet;
    memSQLText: TMemo;
    memRebuildedSQL: TMemo;
    edToken: TEdit;
    lblTokenText: TLabel;
    lblTokenType: TLabel;
    edTokenType: TEdit;
    lblStringDelimiter: TLabel;
    edStringDelimiter: TEdit;
    lblStatementParserClass: TLabel;
    edStatementParserClass: TEdit;
    lblStatementType: TLabel;
    edStatementType: TEdit;
    lblParsedSQL: TLabel;
    memParsedSQL: TMemo;
    btnReset: TButton;
    btnNextToken: TButton;
    btnReplace: TButton;
    btnAll: TButton;
    ActionList1: TActionList;
    actLoadSQL: TAction;
    actSaveSQL: TAction;
    SaveDialog1: TSaveDialog;
    OpenDialog1: TOpenDialog;
    MainMenu1: TMainMenu;
    LoadSQL1: TMenuItem;
    LoadSQL2: TMenuItem;
    SaveSQL1: TMenuItem;
    TreeSheet: TTabSheet;
    TreeView: TTreeView;
    btnBuildParsedTree: TButton;
    procedure btnResetClick(Sender: TObject);
    procedure btnNextTokenClick(Sender: TObject);
    procedure btnAllClick(Sender: TObject);
    procedure btnReplaceClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure actSaveSQLUpdate(Sender: TObject);
    procedure actSaveSQLExecute(Sender: TObject);
    procedure actLoadSQLExecute(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnBuildParsedTreeClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    QueryParser: TgaAdvancedSQLParser;
    procedure SetValues;
    procedure BuildTreeView;
    procedure TokenParsed(Sender: TObject);
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

uses
  gaSelectStm, gaDeleteStm, gaBasicSQLParser, gaSQLParserHelperClasses,
  gaSQLTableRefParsers, TypInfo, gaParserDemoHelpers,
  gaSQLSelectFieldParsers, gaLnkList;

{$R *.DFM}

function GetAliasExpressionStr(stm: TgaSelectSQLStatement; const Alias: string): string;
var
  FieldCur: TgaDoubleListBookmark;
begin
  Result := EmptyStr;
  FieldCur := stm.StatementFields.GetBookmark;
  try
    FieldCur.First;
    while not FieldCur.EOF do
    begin
      if AnsiSameText(TgaSQLSelectField(FieldCur.Item).FieldAlias, Alias) then
      begin
        Result := TgaSQLSelectField(FieldCur.Item).ValueExpression.AsString;
        Exit;
      end;
      FieldCur.Next;
    end;
  finally
    FieldCur.Free;
  end;
end;

procedure TfrmMain.btnResetClick(Sender: TObject);
begin
  QueryParser.SQLText := memSQLText.Lines;
  SetValues;
  BuildTreeView;
end;

procedure TfrmMain.btnNextTokenClick(Sender: TObject);
begin
  QueryParser.NextToken;
  SetValues;
  BuildTreeView;
end;

procedure TfrmMain.btnAllClick(Sender: TObject);
begin
  QueryParser.Reset;
  while not (QueryParser.TokenType = stEnd) do
    QueryParser.NextToken;
  SetValues;
  BuildTreeView;
end;

procedure TfrmMain.btnReplaceClick(Sender: TObject);
var
  stm: TgaSelectSQLStatement;
begin
  QueryParser.SQLText := memSQLText.Lines;
  QueryParser.Reset;
  while not (QueryParser.TokenType = stEnd) do
    QueryParser.NextToken;
  if QueryParser.CurrentStatement is TgaSelectSQLStatement then
  begin
    stm := TgaSelectSQLStatement(QueryParser.CurrentStatement);
    stm.WhereClause.AsString := 'WHERE (SELECT T1.FIELD_NAME FROM TABLE_NAME T1 WHERE T1.ID = T2.ID) = ''TEST''';
    stm := stm.NextUnionPart;
    if stm <> nil then
      stm.WhereClause.AsString := 'WHERE (SELECT T3.FIELD_NAME FROM TABLE_NAME T3 WHERE T3.ID = T4.ID) = ''TEST1''';
  end;
(*  QueryParser.CurrentStatement.AllTables.First;
  while not QueryParser.CurrentStatement.AllTables.Eof do
  begin
    (QueryParser.CurrentStatement.AllTables.CurrentItem as TgaSQLTable).TablePrefix := 'here.athome';
    if (QueryParser.CurrentStatement.AllTables.CurrentItem as TgaSQLTable).IsAliasAllowed then
      (QueryParser.CurrentStatement.AllTables.CurrentItem as TgaSQLTable).TableAlias := 'MeMyself';
    QueryParser.CurrentStatement.AllTables.Next;
  end;
  if QueryParser.CurrentStatement is TgaSelectSQLStatement then
  begin
    if Assigned((QueryParser.CurrentStatement as TgaSelectSQLStatement).OrderByClause) then
      (QueryParser.CurrentStatement as TgaSelectSQLStatement).OrderByClause.AsString := 'order BY Test';
    if Assigned((QueryParser.CurrentStatement as TgaSelectSQLStatement).WhereClause) then
      (QueryParser.CurrentStatement as TgaSelectSQLStatement).WhereClause.AsString := '(Test = ''Test'') and (My = "My")';
  end;
  if QueryParser.CurrentStatement is TgaDeleteSQLStatement then
  begin
    if Assigned((QueryParser.CurrentStatement as TgaDeleteSQLStatement).WhereClause) then
      (QueryParser.CurrentStatement as TgaDeleteSQLStatement).WhereClause.AsString := '(Test = ''Test'') and (My = "My")';
  end;
  QueryParser.CurrentStatement.AllFields.First;
  while not QueryParser.CurrentStatement.AllFields.Eof do
  begin
(*     TgaSQLSelectField(QueryParser.CurrentStatement.AllFields.CurrentItem).FieldName :=
      AnsiUpperCase(TgaSQLSelectField(QueryParser.CurrentStatement.AllFields.CurrentItem).FieldName);
    QueryParser.CurrentStatement.AllFields.Next;
  end; *)
  SetValues;
  BuildTreeView;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  QueryParser := TgaAdvancedSQLParser.Create(Self);
//  QueryParser.OnTokenParsed := TokenParsed;
end;

procedure TfrmMain.TokenParsed(Sender: TObject);
begin
  SetValues;
  BuildTreeView;
end;

type
  THackStm = class(TgaCustomSQLStatement)
  end;

procedure TfrmMain.SetValues;
var
  s1, s2: string;
begin
  edToken.Text := GetReadableString(QueryParser.TokenString);
  edStringDelimiter.Text := QueryParser.StartQuote + QueryParser.EndQuote;
  edTokenType.Text := GetEnumName(TypeInfo(TSQLToken), Ord(QueryParser.TokenType));
  edStatementParserClass.Text := QueryParser.CurrentStatement.ClassName;
  edStatementType.Text := GetEnumName(TypeInfo(TSQLStatementType),
    Ord(QueryParser.CurrentStatement.StatementType));
  memRebuildedSQL.Lines.Text := QueryParser.CurrentStatement.AsString;
  memParsedSQL.Lines.Text := THackStm(QueryParser.CurrentStatement).OriginalSQL.AsString;
  TgaSelectSQLStatement(QueryParser.CurrentStatement).StatementFields.First;
  while not TgaSelectSQLStatement(QueryParser.CurrentStatement).StatementFields.Eof do
  begin
    s1 := TgaSelectSQLStatement(QueryParser.CurrentStatement).StatementFields.CurrentItem.FieldAlias;
    if s1 <> '' then
      s2 := GetAliasExpressionStr(TgaSelectSQLStatement(QueryParser.CurrentStatement), s1)
    else
      s2 := 'No alias';
    memRebuildedSQL.Lines.Add(Format('%s = %s', [s1, s2]));
    TgaSelectSQLStatement(QueryParser.CurrentStatement).StatementFields.Next;
  end;
end;

procedure TfrmMain.actSaveSQLUpdate(Sender: TObject);
begin
  actSaveSQL.Enabled := memSQLText.Lines.Count > 0;
end;                                                   

procedure TfrmMain.actSaveSQLExecute(Sender: TObject);
begin
  if SaveDialog1.Execute then
    memSQLText.Lines.SaveToFile(SaveDialog1.FileName);
end;

procedure TfrmMain.actLoadSQLExecute(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    memSQLText.Lines.LoadFromFile(OpenDialog1.FileName);
    btnResetClick(Self);
  end;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  QueryParser.SQLText := memSQLText.Lines;
  BuildTreeView;
end;

procedure TfrmMain.BuildTreeView;
var
  TreeBuilder: TgaParserTreeBuilder;
begin
  TreeView.Items.BeginUpdate;
  try
    TreeBuilder := TgaParserTreeBuilder.Create(TreeView);
    try
      TreeBuilder.Clear;
      QueryParser.CurrentStatement.AcceptParserVisitor(TreeBuilder);
    finally
      TreeBuilder.Free;
    end;
    if TreeView.Items.Count > 0 then
      TreeView.Items[0].Expand(True);
  finally
    TreeView.Items.EndUpdate;
  end;
end;

procedure TfrmMain.btnBuildParsedTreeClick(Sender: TObject);
begin
  BuildTreeView;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  QueryParser.Free;
end;

end.
