unit gaDoubleListTest;

interface

uses
  gaLnkList, TestFrameWork;

type
  Test_gaDoubleList = class (TgaDoubleList)
  end;
  
  TgaDoubleListTesterBase = class (TTestCase)
  private
    FgaDoubleList: Test_gaDoubleList;
  protected
    property gaDoubleList: Test_gaDoubleList read FgaDoubleList;
  public
    procedure SetUp; override;
    procedure TearDown; override;
  end;
  
  TgaDoubleListBasicTester = class (TgaDoubleListTesterBase)
  published
    procedure VerifyProp_Bof;
    procedure VerifyProp_Count;
    procedure VerifyProp_CurrentItem;
    procedure Verify_Last;
    procedure Verify_Locate;
    procedure Verify_SetCurrentItem;
  end;
  
implementation

{
*************************** TgaDoubleListBasicTester ***************************
}
procedure TgaDoubleListBasicTester.VerifyProp_Bof;
begin
  Check(gaDoubleList.Bof, '#1');
  gaDoubleList.First;
  Check(not gaDoubleList.Bof, '#2');
  gaDoubleList.Last;
  Check(gaDoubleList.Bof, '#3');
  gaDoubleList.Add(nil);
  gaDoubleList.First;
  Check(not gaDoubleList.Bof, '#4');
  gaDoubleList.Previous;
  Check(gaDoubleList.Bof, '#5');
end;

procedure TgaDoubleListBasicTester.VerifyProp_Count;
begin
  CheckEquals(0, gaDoubleList.Count, '#1');
  gaDoubleList.Add(pointer(1));
  CheckEquals(1, gaDoubleList.Count, '#2');
  gaDoubleList.Add(pointer(2));
  CheckEquals(2, gaDoubleList.Count, '#3');
  gaDoubleList.Remove(Pointer(2));
  CheckEquals(1, gaDoubleList.Count, '#4');
  gaDoubleList.Clear;
  CheckEquals(0, gaDoubleList.Count, '#0');
end;

procedure TgaDoubleListBasicTester.VerifyProp_CurrentItem;
begin
  gaDoubleList.Add(pointer(1));
  gaDoubleList.Add(pointer(2));
  gaDoubleList.Add(pointer(3));
  gaDoubleList.Add(pointer(4));
  gaDoubleList.First;
  CheckEquals(1, Integer(gaDoubleList.CurrentItem), '#1');
  gaDoubleList.Next;
  CheckEquals(2, Integer(gaDoubleList.CurrentItem), '#2');
  gaDoubleList.Last;
  CheckEquals(4, Integer(gaDoubleList.CurrentItem), '#3');
  gaDoubleList.Previous;
  CheckEquals(3, Integer(gaDoubleList.CurrentItem), '#4');
end;

procedure TgaDoubleListBasicTester.Verify_Last;
begin
  gaDoubleList.Last;
  Check(gaDoubleList.Bof, 'Not BOF after last in empty list');
  gaDoubleList.Add(Pointer(123));
  gaDoubleList.InsertAfterCurrent(Pointer(223), True);
  gaDoubleList.Last;
  Check(gaDoubleList.Cursor = gaDoubleList.Tail.dllnPrev, 'Last didn''t go to correct item');
end;

procedure TgaDoubleListBasicTester.Verify_Locate;
begin
  gaDoubleList.Add(Pointer(123));
  gaDoubleList.InsertAfterCurrent(Pointer(223), False);
  Check(gaDoubleList.Locate(Pointer(223)), 'Locate(Pointer(223)) failed');
  Check(gaDoubleList.CurrentItem = Pointer(223), 'Locate(Pointer(223)) didn''t locate');
  Check(gaDoubleList.Locate(Pointer(123)), 'Locate(Pointer(123)) failed');
  Check(gaDoubleList.CurrentItem = Pointer(123), 'Locate(Pointer(123)) didn''t locate');
  Check(not gaDoubleList.Locate(Pointer(23)), 'Locate(Pointer(123)) didn''t fail');
end;

procedure TgaDoubleListBasicTester.Verify_SetCurrentItem;
begin
  gaDoubleList.InsertAfterCurrent(nil, True);
  CheckEquals(0, Integer(gaDoubleList.CurrentItem), 'Not nil');
  gaDoubleList.CurrentItem := Pointer(123);
  CheckEquals(123, Integer(gaDoubleList.CurrentItem), 'CurrentItem not set');
end;

{
*************************** TgaDoubleListTesterBase ****************************
}
procedure TgaDoubleListTesterBase.SetUp;
begin
  inherited SetUp;
  FgaDoubleList := Test_gaDoubleList.Create
end;

procedure TgaDoubleListTesterBase.TearDown;
begin
  gaDoubleList.Free;
  inherited TearDown;
end;

initialization
  RegisterTest(TgaDoubleListBasicTester.Suite);

end.
