{*******************************************************}
{                                                       }
{       Advanced SQL statement parser                   }
{       Classes for testing SQL expression parsing      }
{       Copyright (c) 2001 - 2003 AS Gaiasoft           }
{       Created by Gert Kello                           }
{                                                       }
{*******************************************************}

unit gaSQLExpressionParserTests;

interface

uses
  gaSQLParserTestersBase, gaSQLExpressionParsers;

type
  TestgaSQLExpression = class (TgaSQLExpression)
  end;
  
  TgaSQLExpressionParserTester = class (TgaDefaultSQLParserTester)
  private
    FExpression: TestgaSQLExpression;
  protected
    procedure SetUp; override;
    procedure TearDown; override;
    property Expression: TestgaSQLExpression read FExpression;
  published
    procedure TestNestedParens;
    procedure TestSingleMatchedParens;
    procedure TestSingleUnmatchedRightParen;
    procedure TestUnmatchedRightParen;
  end;
  
implementation

{
************************* TgaSQLExpressionParserTester *************************
}
procedure TgaSQLExpressionParserTester.SetUp;
begin
  inherited;
  FExpression := TestgaSQLExpression.Create(Statement);
  Statement.TestedStatementPart := Expression;
end;

procedure TgaSQLExpressionParserTester.TearDown;
begin
  FExpression.Free;
  inherited;
end;

procedure TgaSQLExpressionParserTester.TestNestedParens;
begin
  AddStringToStatement('(', True);
  Check(not Expression.CanParseEnd, 'Parse can be finished after single left paren');
  Check(not Expression.IsInvalid, 'Expression is inavlid after single left paren');
  AddStringToStatement('some strings expression', True);
  Check(not Expression.CanParseEnd, 'Parse can be finished before matching right paren');
  Check(not Expression.IsInvalid, 'Expression is inavlid after "(some strings expression"');
  AddStringToStatement('(', True);
  Check(not Expression.CanParseEnd, 'Parse can be finished after second left paren');
  Check(not Expression.IsInvalid, 'Expression is inavlid after second left paren');
  AddStringToStatement('arr err no --this is in comment: )'#13#10'but this is not: )', True);
  Check(not Expression.CanParseEnd, 'Parse can be finished after first right paren');
  Check(not Expression.IsInvalid, 'Expression is inavlid after first right paren');
  AddStringToStatement('((', True);
  Check(not Expression.CanParseEnd, 'Parse can be finished after fourth left paren');
  Check(not Expression.IsInvalid, 'Expression is inavlid after fourth left paren');
  AddStringToStatement('some info) * some more / & /* some comment )*/)', True);
  Check(not Expression.CanParseEnd, 'Parse can be finished after third right paren');
  Check(not Expression.IsInvalid, 'Expression is inavlid after third right paren');
  AddStringToStatement(')', True);
  Check(Expression.CanParseEnd, 'Parse can not be finished after fourth right paren');
  Check(not Expression.IsInvalid, 'Expression is inavlid after fourth right paren');
  AddStringToStatement(' some more', True);
  Check(Expression.CanParseEnd, 'Parse can not be finished after fourth right paren + some text');
  Check(not Expression.IsInvalid, 'Expression is inavlid after fourth right paren + some text');
  AddStringToStatement('(', True);
  Check(not Expression.CanParseEnd, 'Parse can be finished after final (fifth) left paren');
  Check(not Expression.IsInvalid, 'Expression is inavlid after final (fifth) left paren');
  AddStringToStatement(')', True);
  Check(Expression.CanParseEnd, 'Parse can not be finished after last matching right paren');
  Check(not Expression.IsInvalid, 'Expression invalid after last closing right paren');
  
end;

procedure TgaSQLExpressionParserTester.TestSingleMatchedParens;
begin
  AddStringToStatement('(', True);
  Check(not Expression.CanParseEnd, 'Parse can be finished after single left paren');
  Check(not Expression.IsInvalid, 'Expression is inavlid after single left paren');
  AddStringToStatement('some strings expression', True);
  Check(not Expression.CanParseEnd, 'Parse can be finished before matching right paren');
  Check(not Expression.IsInvalid, 'Expression is inavlid after "(some strings expression"');
  AddStringToStatement(')', True);
  Check(Expression.CanParseEnd, 'Parse can not be finished after matching right paren');
  Check(not Expression.IsInvalid, 'Expression invalid after closing right paren');
end;

procedure TgaSQLExpressionParserTester.TestSingleUnmatchedRightParen;
begin
  AddStringToStatement(')', True);
  Check(Expression.IsInvalid, 'Expression is not invalid after unmatched right paren');
end;

procedure TgaSQLExpressionParserTester.TestUnmatchedRightParen;
begin
  AddStringToStatement('(', True);
  Check(not Expression.CanParseEnd, 'Parse can be finished after single left paren');
  Check(not Expression.IsInvalid, 'Expression is inavlid after single left paren');
  AddStringToStatement('some strings expression', True);
  Check(not Expression.CanParseEnd, 'Parse can be finished before matching right paren');
  Check(not Expression.IsInvalid, 'Expression is inavlid after "(some strings expression"');
  AddStringToStatement(')', True);
  Check(Expression.CanParseEnd, 'Parse can not be finished after matching right paren');
  Check(not Expression.IsInvalid, 'Expression invalid after closing right paren');
  AddStringToStatement(')', True);
  Check(Expression.IsInvalid, 'Expression is not invalid after unmatched right paren');
end;

end.
