{*******************************************************}
{                                                       }
{       Advanced SQL statement parser                   }
{       Classes for testing field reference parsing     }
{       Copyright (c) 2001 - 2003 AS Gaiasoft           }
{       Created by Gert Kello                           }
{                                                       }
{*******************************************************}

unit gaSQLFieldRefTests;

interface

uses
  gaSQLParserTestersBase, gaSQLFieldRefParsers;

type
  TgaSQLFieldReferenceTester = class (TgaDefaultSQLParserTester)
  private
    FFieldRef: TgaSQLFieldReference;
  protected
    procedure FieldNameWithPrefixiesAndComma;
    procedure PlainFieldNameAndComma;
    procedure SetUp; override;
    procedure TearDown; override;
    property FieldRef: TgaSQLFieldReference read FFieldRef;
  published
    procedure AsterixWithPrefixies;
    procedure DoubleQuotedFieldName;
    procedure FieldNameWithPrefixies;
    procedure FieldNameWithPrefixiesAndDelimitier;
    procedure PlainAsterix;
    procedure PlainFieldName;
    procedure PlainFieldNameAndDelimitier;
    procedure QuotedFieldName;
  end;
  
implementation

uses
  TestFrameWork, gaBasicSQLParser, gaAdvancedSQLParser,
  gaSQLParserHelperClasses, SysUtils;

{
************************** TgaSQLFieldReferenceTester **************************
}
procedure TgaSQLFieldReferenceTester.AsterixWithPrefixies;
var
  lToken: TgaSQLTokenObj;
begin
  AddStringToStatement('tere.minu.*', True);
  Statement.CurrentSQL.Last;
  Statement.CurrentSQL.Previous;
  lToken := Statement.CurrentSQL.CurrentItem;
  Check(FieldRef.ParseComplete, 'Fieldref parse not completed after "tere.minu.*"');
  Check(not FieldRef.IsInvalid, 'Fieldref invalid "tere.minu.*"');
  Check(FieldRef.FirstItem = Statement.CurrentSQL.FirstItem, 'Fieldref first token wrong');
  Check(FieldRef.LastItem = Statement.CurrentSQL.LastItem, 'Fieldref last token wrong');
  Check(FieldRef.AsString = 'tere.minu.*', 'Fieldref <> "tere.minu.*"');
  Check(FieldRef.FieldName = '*', 'FieldRef.FieldName <> "*"');
  Check(FieldRef.FieldPrefixies.FirstItem = Statement.CurrentSQL.FirstItem, 'FieldPrefixies.FirstItem wrong');
  Check(FieldRef.FieldPrefixies.LastItem = lToken, 'FieldPrefixies.LastItem wrong');
  Check(not FieldRef.FieldPrefixies.IsEmpty, 'FieldPrefixies list is empty');
  Check(FieldRef.FieldPrefix = 'tere.minu.', Format('Unexpected field prefixies: "%s"', [FieldRef.FieldPrefix]));
end;

procedure TgaSQLFieldReferenceTester.DoubleQuotedFieldName;
begin
  AddStringToStatement('"TestField"', True);
  Check(not FieldRef.ParseComplete, 'Parse complete without comma/delimitier');
  FieldRef.ParseComplete := True;
  Check(FieldRef.ParseComplete, 'Parse not complete after explicit FieldRef.ParseComplete := True');
  Check(not FieldRef.IsInvalid, 'Fieldref invalid after "TestField"');
  Check(FieldRef.FirstItem = Statement.CurrentSQL.FirstItem, 'Fieldref first token wrong');
  Check(FieldRef.LastItem = Statement.CurrentSQL.LastItem, 'Fieldref last token wrong');
  Check(FieldRef.FieldName = '"TestField"', 'Wrong FieldRef.FieldName');
  Check(FieldRef.FieldPrefix = '', 'FieldRef.FieldPrefix <> ''''');
  Check(FieldRef.FieldPrefixies.IsEmpty, 'not FieldRef.FieldPrefixies.IsEmpty');
  Check(FieldRef.AsString = '"TestField"', 'FieldRef.AsString <> "TestField"');
  Check(FieldRef.FieldPrefix = '', Format('Unexpected field prefixies: "%s"', [FieldRef.FieldPrefix]));
end;

procedure TgaSQLFieldReferenceTester.FieldNameWithPrefixies;
var
  lToken: TgaSQLTokenObj;
begin
  AddStringToStatement('tere.minu.field', True);
  Check(not FieldRef.ParseComplete, 'Parse complete without comma/delimitier');
  FieldRef.ParseComplete := True;
  Check(FieldRef.ParseComplete, 'Parse not complete after explicit FieldRef.ParseComplete := True');
  Statement.CurrentSQL.Last;
  Statement.CurrentSQL.Previous;
  lToken := Statement.CurrentSQL.CurrentItem;
  Check(FieldRef.ParseComplete, 'Fieldref parse not completed after "tere.minu.field"');
  Check(not FieldRef.IsInvalid, 'Fieldref invalid "tere.minu.field"');
  Check(FieldRef.FirstItem = Statement.CurrentSQL.FirstItem, 'Fieldref first token wrong');
  Check(FieldRef.LastItem = Statement.CurrentSQL.LastItem, 'Fieldref last token wrong');
  Check(FieldRef.AsString = 'tere.minu.field', 'Fieldref <> "tere.minu.field"');
  Check(FieldRef.FieldName = 'field', 'FieldRef.FieldName <> "field"');
  Check(FieldRef.FieldPrefixies.FirstItem = Statement.CurrentSQL.FirstItem, 'FieldPrefixies.FirstItem wrong');
  Check(FieldRef.FieldPrefixies.LastItem = lToken, 'FieldPrefixies.LastItem wrong');
  Check(not FieldRef.FieldPrefixies.IsEmpty, 'FieldPrefixies list is empty');
  Check(FieldRef.FieldPrefix = 'tere.minu.', Format('Unexpected field prefixies: "%s"', [FieldRef.FieldPrefix]));
end;

procedure TgaSQLFieldReferenceTester.FieldNameWithPrefixiesAndComma;
begin
  AddStringToStatement('tere.minu.field,', True);
  Check(FieldRef.ParseComplete, 'Parse not complete with comma');
  Check(not FieldRef.IsInvalid, 'Fieldref invalid "tere.minu.field"');
  Check(FieldRef.FirstItem = Statement.CurrentSQL.FirstItem, 'Fieldref first token wrong');
  Statement.CurrentSQL.Last;
  Statement.CurrentSQL.Previous;
  Check(FieldRef.LastItem = Statement.CurrentSQL.CurrentItem, 'Fieldref last token wrong');
  Statement.CurrentSQL.Previous;
  Check(FieldRef.FieldPrefixies.FirstItem = Statement.CurrentSQL.FirstItem, 'FieldPrefixies.FirstItem wrong');
  Check(FieldRef.FieldPrefixies.LastItem = Statement.CurrentSQL.CurrentItem, 'FieldPrefixies.LastItem wrong');
  Check(FieldRef.AsString = 'tere.minu.field', 'Fieldref <> "tere.minu.field"');
  Check(FieldRef.FieldName = 'field', 'FieldRef.FieldName <> "field"');
  Check(not FieldRef.FieldPrefixies.IsEmpty, 'FieldPrefixies list is empty');
  Check(FieldRef.FieldPrefix = 'tere.minu.', Format('Unexpected field prefixies: "%s"', [FieldRef.FieldPrefix]));
end;

procedure TgaSQLFieldReferenceTester.FieldNameWithPrefixiesAndDelimitier;
begin
  AddStringToStatement('tere.minu.field ', False);
  Check(not FieldRef.ParseComplete, 'Parse complete after delimitier');
  FieldRef.ParseComplete := True;
  Check(not FieldRef.IsInvalid, 'Fieldref invalid "tere.minu.field"');
  Check(FieldRef.FirstItem = Statement.CurrentSQL.FirstItem, 'Fieldref first token wrong');
  Statement.CurrentSQL.Last;
  while Statement.CurrentSQL.CurrentItem.TokenType = stDelimitier do
    Statement.CurrentSQL.Previous;
  Check(FieldRef.LastItem = Statement.CurrentSQL.CurrentItem, 'Fieldref last token wrong');
  Statement.CurrentSQL.Previous;
  Check(FieldRef.FieldPrefixies.FirstItem = Statement.CurrentSQL.FirstItem, 'FieldPrefixies.FirstItem wrong');
  Check(FieldRef.FieldPrefixies.LastItem = Statement.CurrentSQL.CurrentItem, 'FieldPrefixies.LastItem wrong');
  Check(FieldRef.AsString = 'tere.minu.field', 'Fieldref <> "tere.minu.field"');
  Check(FieldRef.FieldName = 'field', 'FieldRef.FieldName <> "field"');
  Check(not FieldRef.FieldPrefixies.IsEmpty, 'FieldPrefixies list is empty');
  Check(FieldRef.FieldPrefix = 'tere.minu.', Format('Unexpected field prefixies: "%s"', [FieldRef.FieldPrefix]));
end;

procedure TgaSQLFieldReferenceTester.PlainAsterix;
begin
  AddStringToStatement('*', True);
  Check(FieldRef.ParseComplete, 'Fieldref parse not completed after "*"');
  Check(not FieldRef.IsInvalid, 'Fieldref invalid after "*"');
  Check(FieldRef.FirstItem = Statement.CurrentSQL.FirstItem, 'Fieldref first token wrong');
  Check(FieldRef.LastItem = Statement.CurrentSQL.LastItem, 'Fieldref last token wrong');
  Check(FieldRef.AsString = '*', Format('Fieldref <> "*" ("%S")', [FieldRef.AsString]));
  Check(FieldRef.FieldName = '*', 'Wrong FieldRef.FieldName');
  Check(FieldRef.FieldPrefixies.IsEmpty, 'FieldPrefixies list is not empty');
  Check(FieldRef.FieldPrefix = '', Format('Unexpected field prefixies: "%s"', [FieldRef.FieldPrefix]));
end;

procedure TgaSQLFieldReferenceTester.PlainFieldName;
begin
  AddStringToStatement('TestField', True);
  Check(not FieldRef.ParseComplete, 'Parse complete without comma/delimitier');
  FieldRef.ParseComplete := True;
  Check(FieldRef.ParseComplete, 'Parse not complete after explicit FieldRef.ParseComplete := True');
  Check(not FieldRef.IsInvalid, 'Fieldref invalid after "TestField"');
  Check(FieldRef.FirstItem = Statement.CurrentSQL.FirstItem, 'Fieldref first token wrong');
  Check(FieldRef.LastItem = Statement.CurrentSQL.LastItem, 'Fieldref last token wrong');
  Check(FieldRef.FieldName = 'TestField', 'Wrong FieldRef.FieldName');
  Check(FieldRef.FieldPrefix = '', 'FieldRef.FieldPrefix <> ''''');
  Check(FieldRef.FieldPrefixies.IsEmpty, 'not FieldRef.FieldPrefixies.IsEmpty');
  Check(FieldRef.AsString = 'TestField', 'FieldRef.AsString <> "TestField"');
  Check(FieldRef.FieldPrefix = '', Format('Unexpected field prefixies: "%s"', [FieldRef.FieldPrefix]));
end;

procedure TgaSQLFieldReferenceTester.PlainFieldNameAndComma;
begin
  AddStringToStatement('TestField,', True);
  Check(FieldRef.ParseComplete, 'Parse not complete with delimitier');
  Check(not FieldRef.IsInvalid, 'Fieldref invalid after "TestField"');
  Check(FieldRef.FirstItem = Statement.CurrentSQL.FirstItem, 'Fieldref first token wrong');
  Statement.CurrentSQL.Last;
  Statement.CurrentSQL.Previous;
  Check(FieldRef.LastItem = Statement.CurrentSQL.CurrentItem, 'Fieldref last token wrong');
  Check(FieldRef.FieldName = 'TestField', 'Wrong FieldRef.FieldName');
  Check(FieldRef.FieldPrefix = '', 'FieldRef.FieldPrefix <> ''''');
  Check(FieldRef.FieldPrefixies.IsEmpty, 'not FieldRef.FieldPrefixies.IsEmpty');
  Check(FieldRef.AsString = 'TestField', 'FieldRef.AsString <> "TestField"');
  Check(FieldRef.FieldPrefix = '', Format('Unexpected field prefixies: "%s"', [FieldRef.FieldPrefix]));
end;

procedure TgaSQLFieldReferenceTester.PlainFieldNameAndDelimitier;
begin
  AddStringToStatement('TestField ', False);
  Check(not FieldRef.ParseComplete, 'Parse complete after delimitier');
  FieldRef.ParseComplete := True;
  Check(not FieldRef.IsInvalid, 'Fieldref invalid after "TestField"');
  Check(FieldRef.FirstItem = Statement.CurrentSQL.FirstItem, 'Fieldref first token wrong');
  Statement.CurrentSQL.Last;
  while Statement.CurrentSQL.CurrentItem.TokenType = stDelimitier do
    Statement.CurrentSQL.Previous;
  Check(FieldRef.LastItem = Statement.CurrentSQL.CurrentItem, 'Fieldref last token wrong');
  Check(FieldRef.FieldName = 'TestField', 'Wrong FieldRef.FieldName');
  Check(FieldRef.FieldPrefix = '', 'FieldRef.FieldPrefix <> ''''');
  Check(FieldRef.FieldPrefixies.IsEmpty, 'not FieldRef.FieldPrefixies.IsEmpty');
  Check(FieldRef.AsString = 'TestField', 'FieldRef.AsString <> "TestField"');
  Check(FieldRef.FieldPrefix = '', Format('Unexpected field prefixies: "%s"', [FieldRef.FieldPrefix]));
end;

procedure TgaSQLFieldReferenceTester.QuotedFieldName;
begin
  StartExpectingException(EgaSQLInvalidTokenEncountered);
  AddStringToStatement('''TestField''', True);
  StopExpectingException('FieldRef accepted strin ''TestField''');
end;

procedure TgaSQLFieldReferenceTester.SetUp;
begin
  inherited;
  FFieldRef := TgaSQLFieldReference.Create(Statement);
  Statement.TestedStatementPart := FieldRef;
end;

procedure TgaSQLFieldReferenceTester.TearDown;
begin
  FFieldRef.Free;
  inherited;
end;


end.
