{*******************************************************}
{                                                       }
{       Advanced SQL statement parser                   }
{       Classes for testing field reference parsing     }
{       Copyright (c) 2004 Gert Kello                   }
{       Created by Gert Kello                           }
{                                                       }
{*******************************************************}

unit gaSQLOrderByTests;

interface

uses
  gaSQLParserTestersBase, gaSQLFieldRefParsers;


type
  TgaSQLOrderByTester = class (TgaDefaultSQLParserTester)
  private
    FOrderByRef: TgaSQLOrderByReference;
  protected
    procedure SetUp; override;
    procedure TearDown; override;
    property OrderByRef: TgaSQLOrderByReference read FOrderByRef;
  published
    procedure TestOrderNullFirst;
    procedure TestSetNullsHandling;
  end;

implementation

uses
  SysUtils;

{ TgaSQLOrderByTester }

procedure TgaSQLOrderByTester.SetUp;
begin
  inherited;
  FOrderByRef := TgaSQLOrderByReference.Create(Statement);
  Statement.TestedStatementPart := OrderByRef;
end;

procedure TgaSQLOrderByTester.TearDown;
begin
  FOrderByRef.Free;
  inherited;
end;

procedure TgaSQLOrderByTester.TestOrderNullFirst;
begin
  AddStringToStatement('myref NULLS first', True);
  OrderByRef.Last;
  OrderByRef.ParseComplete := True;
  Check(OrderByRef.NullHandling = onhNullsFirst);
  OrderByRef.NullHandling := onhNullLast;
  Check(AnsiSameText('myref NULLS LAST', OrderByRef.AsString));
  OrderByRef.NullHandling := onhNotSpecified;
  Check(AnsiSameText('myref', Trim(OrderByRef.AsString)));
end;

procedure TgaSQLOrderByTester.TestSetNullsHandling;
begin
  AddStringToStatement('myref', True);
  OrderByRef.Last;
  OrderByRef.ParseComplete := True;
  OrderByRef.NullHandling := onhNullLast;
  Check(AnsiSameText('myref NULLS LAST', OrderByRef.AsString));
  OrderByRef.NullHandling := onhNotSpecified;
  Check(AnsiSameText('myref', Trim(OrderByRef.AsString)));
end;

end.
