{*******************************************************}
{                                                       }
{       Advanced SQL statement parser                   }
{       Base classes for parser testers                 }
{       Copyright (c) 2001 - 2003 AS Gaiasoft           }
{       Created by Gert Kello                           }
{                                                       }
{*******************************************************}

unit gaSQLParserTestersBase;

interface

uses
  TestFrameWork, gaBasicSQLParser, gaAdvancedSQLParser,
  gaSQLParserHelperClasses;

type
  TgaTestingStatement = class (TgaCustomSQLStatement)
  private
    FTestedStatementPart: TgaSQLStatementPart;
  protected
    function GetStatementType: TSQLStatementType; override;
    procedure ModifyStatementInNormalState(Sender: TObject; AToken: 
            TgaSQLTokenObj); override;
  public
    property TestedStatementPart: TgaSQLStatementPart read FTestedStatementPart 
            write FTestedStatementPart;
  end;
  
  TgaSQLParserTesterBase = class (TTestCase)
  private
    FStatement: TgaCustomSQLStatement;
    FTokenList: TgaSQLTokenHolderList;
  protected
    procedure AddStringToStatement(const AString: string; TrimTokens: Boolean);
    procedure AddToken(AToken: TgaSQLTokenObj);
    function CreateTestStatement: TgaCustomSQLStatement; virtual; abstract;
    procedure SetUp; override;
    procedure TearDown; override;
    property Statement: TgaCustomSQLStatement read FStatement write FStatement;
    property TokenList: TgaSQLTokenHolderList read FTokenList;
  end;
  
  TgaDefaultSQLParserTester = class (TgaSQLParserTesterBase)
  private
    function GetStatement: TgaTestingStatement;
  protected
    function CreateTestStatement: TgaCustomSQLStatement; override;
  public
    property Statement: TgaTestingStatement read GetStatement;
  end;
  
implementation

{
***************************** TgaTestingStatement ******************************
}
function TgaTestingStatement.GetStatementType: TSQLStatementType;
begin
  Result := sstUnknown;
end;

procedure TgaTestingStatement.ModifyStatementInNormalState(Sender: TObject; 
        AToken: TgaSQLTokenObj);
begin
  inherited;
  if TestedStatementPart <> nil then
    TestedStatementPart.ExecuteTokenAdded(Sender, AToken);
end;

{
**************************** TgaSQLParserTesterBase ****************************
}
procedure TgaSQLParserTesterBase.AddStringToStatement(const AString: string; 
        TrimTokens: Boolean);
begin
  ParseStringToTokens(AString, TokenList);
  if TrimTokens then
    TrimTokenList(TokenList, True);
  TokenList.First;
  while not TokenList.Eof do
  begin
    AddToken(TokenList.CurrentItem);
    TokenList.Next;
  end;
end;

procedure TgaSQLParserTesterBase.AddToken(AToken: TgaSQLTokenObj);
begin
  Statement.CurrentSQL.AddToken(AToken);
  TgaTestingStatement(Statement).CurrentToken := AToken;
  Statement.DoTokenAdded(Self, AToken);
end;

procedure TgaSQLParserTesterBase.SetUp;
begin
  inherited;
  FStatement := CreateTestStatement;
  FTokenList := TgaSQLTokenHolderList.Create(nil);
end;

procedure TgaSQLParserTesterBase.TearDown;
begin
  FTokenList.Free;
  FStatement.Free;
  inherited;
end;

{
************************** TgaDefaultSQLParserTester ***************************
}
function TgaDefaultSQLParserTester.CreateTestStatement: TgaCustomSQLStatement;
begin
  Result := TgaTestingStatement.Create(nil);
end;

function TgaDefaultSQLParserTester.GetStatement: TgaTestingStatement;
begin
  Result := (inherited Statement) as TgaTestingStatement;
end;


end.
