{*******************************************************}
{                                                       }
{       Advanced SQL statement parser                   }
{       Classes for testing self found bugs             }
{       Copyright (c) 2001 - 2003 AS Gaiasoft           }
{       Created by Gert Kello                           }
{                                                       }
{*******************************************************}

unit gaSelfFoundBugsTests;

interface

uses
  gaSQLParserTestersBase, gaBasicSQLParser, gaAdvancedSQLParser, TestFrameWork,
  gaAdvancedParserTestersBase, Forms;

type
  TgaSelfFoundBugs = class (TgaStatementTesterBase)
  published
    procedure ComplexMyykSQL;
    procedure DelimiterStartedSelect;
    procedure FieldRefParseNotTerminatedByOperator;
    procedure FunctionAsFieldException;
    procedure NestedFuntions;
    procedure OtherQuery;
    procedure SomeQuery;
    procedure SubSelectInWhereClause;
    procedure TestX;
  end;
  

implementation

{
******************************* TgaSelfFoundBugs *******************************
}
procedure TgaSelfFoundBugs.ComplexMyykSQL;
begin
  ParseStatement('SELECT L.DOK_LIIK DOK_LIIK, L.KASSAORD_S KASSAORD_S, L.DOk_NR DOK_NR,'#13#10 +
                  'L.KUUP KUUP, S.SUMMA SUMMA, S.UNIKKOOD UNIKKOOD'#13#10 +
                  'FROM LAEKUM L, LAEKSISU S'#13#10 +
                  'WHERE (L.LAEKUNIK = S.LAEKUNIK) AND'#13#10 +
                  '(S.UNIKKOOD IN ('#13#10 +
                  'SELECT UNIKKOOD'#13#10 +
                  'FROM MYYKARVE A'#13#10 +
                  'WHERE'#13#10 +
                  '((A.ARVEOLEK IS NOT NULL) OR (A.ARVEOLEK IS NULL))'#13#10 +
                  'AND (A.KUUP >= :algus) AND (A.KUUP <= :lopp)'#13#10 +
                  '))'#13#10 +
                  'ORDER BY S.UNIKKOOD, L.KUUP');
  Check(Statement.AllTables.Count = 3, 'Wrong AllTables count');
  Statement.AllTables.First;
  Check(Statement.AllTables.CurrentItem.AsString = 'LAEKUM L', 'Wrong first table');
  Statement.AllTables.Next;
  Check(Statement.AllTables.CurrentItem.AsString = 'LAEKSISU S', 'Wrong second table');
  Statement.AllTables.Next;
  Check(Statement.AllTables.CurrentItem.AsString = 'MYYKARVE A', 'Wrong third table');
end;

procedure TgaSelfFoundBugs.DelimiterStartedSelect;
begin
  ParseStatement(' select * from test');
end;

procedure TgaSelfFoundBugs.FieldRefParseNotTerminatedByOperator;
begin
  ParseStatement('select * from myykarve where (number_<>0) order by Aasta, Seeria, Number_');
end;

procedure TgaSelfFoundBugs.FunctionAsFieldException;
begin
  ParseStatement('SELECT MAX(UNIKKOOD) FROM myykarve');
end;

procedure TgaSelfFoundBugs.NestedFuntions;
begin
  ParseStatement('Select * from test where trim(upper(Kood))= :ukood ');
end;

procedure TgaSelfFoundBugs.OtherQuery;
begin
  ParseStatement('SELECT KUUP FROM LAEKUM'#$D#$A'WHERE LAEKUNIK IN ('#$D#$A'SELECT MAX(LAEKUNIK) FROM LAEKUM'#$D#$A')'#$D#$A);
end;

procedure TgaSelfFoundBugs.SomeQuery;
begin
  ParseStatement('SELECT UNIKKOOD FROM MYYKARVE'#$D#$A'WHERE (AASTA = :aasta) AND (NUMBER_ = :number_) AND '#$D#$A'( (SEERIA IS NULL) OR (SEERIA = :seeria))'#$D#$A);
end;

procedure TgaSelfFoundBugs.SubSelectInWhereClause;
begin
  ParseStatement('select * from test where a < (Select u from test2)');
  Check(Statement.AllTables.Count = 2, 'Wrong AllTables count');
  Statement.AllTables.First;
  Check(Statement.AllTables.CurrentItem.AsString = 'test', 'Wrong first table');
  Statement.AllTables.Next;
  Check(Statement.AllTables.CurrentItem.AsString = 'test2', 'Wrong second table');
end;

procedure TgaSelfFoundBugs.TestX;
begin
  ParseStatement('SELECT SEERIA FROM MYYKARVE'#$D#$A'WHERE (UNIKKOOD IN (SELECT MAX(UNIKKOOD) FROM MYYKARVE))'#$D#$A);
  Check(Statement.AllTables.Count = 2, 'Wrong AllTables count');
  Statement.AllTables.First;
  Check(Statement.AllTables.CurrentItem.AsString = 'MYYKARVE', 'Wrong first table');
  Statement.AllTables.Next;
  Check(Statement.AllTables.CurrentItem.AsString = 'MYYKARVE', 'Wrong second table');
end;


end.
