unit gaSharedDoubleListTests;

interface

uses
  gaLnkList, TestFrameWork;

type
  Test_gaSharedDoubleList = class (TgaSharedDoubleList)
  end;
  
  TgaSharedDoubleListTesterBase = class (TTestCase)
  private
    FgaMasterList: Test_gaSharedDoubleList;
    FgaMirrorList: Test_gaSharedDoubleList;
    FgaSharedDoubleList: Test_gaSharedDoubleList;
  protected
    property gaMasterList: Test_gaSharedDoubleList read FgaMasterList;
    property gaMirrorList: Test_gaSharedDoubleList read FgaMirrorList;
    property gaSharedDoubleList: Test_gaSharedDoubleList read 
            FgaSharedDoubleList;
  public
    procedure SetUp; override;
    procedure TearDown; override;
  end;
  
  TgaSharedDoubleListBasicTester = class (TgaSharedDoubleListTesterBase)
  published
    procedure VerifyProp_DataOwner;
    procedure VerifyProp_IsDataOwner;
    procedure VerifyProp_StrictEndPos;
    procedure VerifyProp_StrictStartPos;
    procedure Verify_AddDataUser;
    procedure Verify_DataOwnerDestroyed;
    procedure Verify_SetEndPos;
    procedure Verify_SetHeadNode;
    procedure Verify_SetStartPos;
    procedure Verify_SetTailNode;
  end;
  

implementation

{
************************ TgaSharedDoubleListTesterBase *************************
}
procedure TgaSharedDoubleListTesterBase.SetUp;
begin
  inherited SetUp;
  FgaMasterList := Test_gaSharedDoubleList.Create
end;

procedure TgaSharedDoubleListTesterBase.TearDown;
begin
  gaMasterList.Free;
  inherited TearDown;
end;

{
************************ TgaSharedDoubleListBasicTester ************************
}
procedure TgaSharedDoubleListBasicTester.VerifyProp_DataOwner;
var
  SecMirList: Test_gaSharedDoubleList;
begin
  SecMirList := nil;
  try
    CheckSame(gaMasterList.DataOwner, gaMasterList, 'gaMasterList.DataOwner <> gaMasterList');
    FgaMirrorList := Test_gaSharedDoubleList.CreateMirror(gaMasterList);
    CheckSame(gaMirrorList.DataOwner, gaMasterList, 'gaMirrorList.DataOwner <> gaMasterList');
    SecMirList := Test_gaSharedDoubleList.CreateMirror(gaMasterList);
    CheckSame(SecMirList.DataOwner, gaMasterList, 'SecMirList.DataOwner <> gaMasterList');
  finally
    SecMirList.Free;
    FgaMirrorList.Free;
    FgaMirrorList := nil;
  end;
end;

procedure TgaSharedDoubleListBasicTester.VerifyProp_IsDataOwner;
begin
  Check(gaMasterList.IsDataOwner, 'not gaMasterList.isDataOwner');
  FgaMirrorList := Test_gaSharedDoubleList.CreateMirror(gaMasterList);
  try
    Check(not gaMirrorList.isDataOwner, 'gaMirrorList.isDataOwner');
  finally
    FgaMirrorList.Free;
    FgaMirrorList := nil;
  end;
end;

procedure TgaSharedDoubleListBasicTester.VerifyProp_StrictEndPos;
begin
  FgaMirrorList := Test_gaSharedDoubleList.CreateMirror(gaMasterList);
  try
    gaMasterList.Add(Pointer(1));
    gaMasterList.Add(Pointer(2));
    Check(gaMirrorList.Locate(Pointer(2)), '2nd item not found in mirror list (strictEnd = False)');
    gaMasterList.Last;
    gaMirrorList.SetEndPos(gaMasterList, True);
    Check(gaMirrorList.StrictEndPos, 'StrictEndPos not stored');
    gaMasterList.InsertAfterCurrent(Pointer(3), True);
    Check(not gaMirrorList.Locate(Pointer(3)), '3rd item found in mirror list (strictEnd = True)');
    gaMirrorList.Add(Pointer(4));
    Check(gaMirrorList.Locate(Pointer(4)), '4th item (inserted into mirror) not found in mirror list (strictEnd = True)');
    gaMirrorList.Last;
    gaMirrorList.Previous;
    gaMirrorList.SetEndPos(gaMirrorList, False);
    Check(not gaMirrorList.StrictEndPos, '2: StrictEndPos not stored');
    gaMasterList.Locate(gaMirrorList.CurrentItem);
    gaMasterList.InsertAfterCurrent(Pointer(5), False);
    Check(gaMirrorList.Locate(Pointer(5)), '5th item (inserted into mirror) not found in mirror list (strictEnd = True)')
  finally
    FgaMirrorList.Free;
    FgaMirrorList := nil;
  end;
end;

procedure TgaSharedDoubleListBasicTester.VerifyProp_StrictStartPos;
begin
  FgaMirrorList := Test_gaSharedDoubleList.CreateMirror(gaMasterList);
  try
    gaMasterList.Add(Pointer(1));
    gaMasterList.Add(Pointer(2));
    Check(gaMirrorList.Locate(Pointer(1)), '2nd item not found in mirror list (strictEnd = False)');
    gaMasterList.First;
    gaMirrorList.SetStartPos(gaMasterList, True);
    Check(gaMirrorList.StrictStartPos, 'StrictStartPos not stored');
    gaMasterList.First;
    gaMasterList.Previous;
    check(gaMasterList.Bof, 'not gaMasterList.Bof');
    gaMasterList.InsertAfterCurrent(Pointer(3), false);
    Check(not gaMirrorList.Locate(Pointer(3)), '3rd item found in mirror list (strictStart = True)');
    gaMirrorList.Add(Pointer(4));
    Check(gaMirrorList.Locate(Pointer(4)), '4th item (inserted into mirror) not found in mirror list (strictStart = True)');
    gaMirrorList.First;
    gaMirrorList.Next;
    gaMirrorList.SetStartPos(gaMirrorList, False);
    Check(not gaMirrorList.StrictStartPos, '2: StrictStartPos not stored');
    gaMasterList.Locate(gaMirrorList.CurrentItem);
    gaMasterList.Previous;
    gaMasterList.InsertAfterCurrent(Pointer(5), False);
    Check(gaMirrorList.Locate(Pointer(5)), '5th item (inserted into mirror) not found in mirror list (strictStart = True)')
  finally
    FgaMirrorList.Free;
    FgaMirrorList := nil;
  end;
end;

procedure TgaSharedDoubleListBasicTester.Verify_AddDataUser;
begin
  FgaMirrorList := Test_gaSharedDoubleList.CreateMirror(gaMasterList);
  try
    StartExpectingException(EgaLinkListError);
    FgaMirrorList.AddDataUser(gaMasterList, npFirstPass);
    StopExpectingException('No exception when removing data user from mirroring list');
  finally
    FgaMirrorList.Free;
    FgaMirrorList := nil;
  end;
end;

procedure TgaSharedDoubleListBasicTester.Verify_DataOwnerDestroyed;
begin
  gaMasterList.Add(Pointer(1));
  gaMasterList.Add(Pointer(2));
  gaMasterList.Add(Pointer(3));
  gaMasterList.Add(Pointer(4));
  gaMasterList.Add(Pointer(5));
  FgaMirrorList := Test_gaSharedDoubleList.CreateMirror(gaMasterList);
  try
    FgaMasterList.Free;
    Check(gaMirrorList.DataOwner = nil);
  finally
    FgaMasterList := nil;
    FgaMirrorList.Free;
    FgaMirrorList := nil;
  end;
end;

procedure TgaSharedDoubleListBasicTester.Verify_SetEndPos;
begin
  gaMasterList.Add(Pointer(1));
  gaMasterList.Add(Pointer(2));
  gaMasterList.Add(Pointer(3));
  FgaMirrorList := Test_gaSharedDoubleList.CreateMirror(gaMasterList);
  try
    CheckEquals(3, Integer(gaMirrorList.LastItem), '1: gaMirrorList.LastItem');
    gaMirrorList.First;
    gaMirrorList.SetEndPos(gaMirrorList, True);
    Check(gaMirrorList.IsCursorCorrect, 'Cursor not correct');
    CheckEquals(1, Integer(gaMirrorList.LastItem), '2: gaMirrorList.LastItem');
    gaMasterList.First;
    gaMasterList.Previous;
    gaMirrorList.SetEndPos(gaMasterList, True);
    Check(gaMirrorList.IsCursorCorrect, 'Cursor not correct');
    Check(gaMirrorList.IsEmpty, 'not gaMirrorList.IsEmpty');
    gaMasterList.Last;
    gaMasterList.Previous;
    gaMirrorList.SetEndPos(gaMasterList, True);
    Check(gaMirrorList.IsCursorCorrect, 'Cursor not correct');
    CheckEquals(2, Integer(gaMirrorList.LastItem), '3: gaMirrorList.LastItem');
  finally
    FgaMirrorList.Free;
    FgaMirrorList := nil;
  end;
end;

procedure TgaSharedDoubleListBasicTester.Verify_SetHeadNode;
begin
  gaMasterList.Add(Pointer(1));
  gaMasterList.Add(Pointer(2));
  gaMasterList.Last;
  FgaMirrorList := Test_gaSharedDoubleList.CreateMirror(gaMasterList);
  try
    gaMirrorList.Last;
    Check(gaMasterList.Head.dllnNotifiers <> nil, 'gaMasterList.Head.dllnNotifiers = nil');
    Check(gaMasterList.Head.dllnNotifiers.dllRequester = gaMirrorList, 'gaMasterList.Head.dllnNotifiers.dllRequester <> gaMirrorList');
    FgaMirrorList.SetHeadNode(gaMasterList.Head.dllnNext);
    Check(gaMasterList.Head.dllnNotifiers = nil, 'gaMasterList.Head.dllnNotifiers <> nil');
    Check(gaMasterList.Head.dllnNext.dllnNotifiers <> nil, 'gaMasterList.Head.dllnNext.dllnNotifiers = nil');
    Check(gaMasterList.Head.dllnNext.dllnNotifiers.dllRequester = gaMirrorList, 'gaMasterList.Head.dllnNext.dllnNotifiers.dllRequester <> gaMirrorList');
  finally
    FgaMirrorList.Free;
    FgaMirrorList := nil;
  end;
end;

procedure TgaSharedDoubleListBasicTester.Verify_SetStartPos;
begin
  gaMasterList.Add(Pointer(1));
  gaMasterList.Add(Pointer(2));
  gaMasterList.Add(Pointer(3));
  FgaMirrorList := Test_gaSharedDoubleList.CreateMirror(gaMasterList);
  try
    CheckEquals(1, Integer(gaMirrorList.FirstItem), '1: gaMirrorList.FirstItem');
    gaMirrorList.Last;
    gaMirrorList.SetStartPos(gaMirrorList, True);
    Check(gaMirrorList.IsCursorCorrect, 'Cursor not correct');
    CheckEquals(3, Integer(gaMirrorList.FirstItem), '2: gaMirrorList.FirstItem');
    gaMasterList.Last;
    gaMasterList.Next;
    gaMirrorList.SetStartPos(gaMasterList, True);
    Check(gaMirrorList.IsCursorCorrect, 'Cursor not correct');
    Check(gaMirrorList.IsEmpty, 'not gaMirrorList.IsEmpty');
    gaMasterList.First;
    gaMasterList.Next;
    gaMirrorList.SetStartPos(gaMasterList, True);
    Check(gaMirrorList.IsCursorCorrect, 'Cursor not correct');
    CheckEquals(2, Integer(gaMirrorList.FirstItem), '3: gaMirrorList.FirstItem');
  finally
    FgaMirrorList.Free;
    FgaMirrorList := nil;
  end;
end;

procedure TgaSharedDoubleListBasicTester.Verify_SetTailNode;
begin
  gaMasterList.Add(Pointer(1));
  FgaMirrorList := Test_gaSharedDoubleList.CreateMirror(gaMasterList);
  try
    Check(gaMasterList.Tail.dllnNotifiers <> nil, 'gaMasterList.Tail.dllnNotifiers = nil');
    Check(gaMasterList.Tail.dllnNotifiers.dllRequester = gaMirrorList, 'gaMasterList.Tail.dllnNotifiers.dllRequester <> gaMirrorList');
    FgaMirrorList.SetTailNode(gaMasterList.Tail.dllnPrev);
    Check(gaMasterList.Tail.dllnNotifiers = nil, 'gaMasterList.Tail.dllnNotifiers <> nil');
    Check(gaMasterList.Tail.dllnPrev.dllnNotifiers <> nil, 'gaMasterList.Tail.dllnPrev.dllnNotifiers = nil');
    Check(gaMasterList.Tail.dllnPrev.dllnNotifiers.dllRequester = gaMirrorList, 'gaMasterList.Tail.dllnPrev.dllnNotifiers.dllRequester <> gaMirrorList');
  finally
    FgaMirrorList.Free;
    FgaMirrorList := nil;
  end;
end;

initialization
  RegisterTest(TgaSharedDoubleListBasicTester.Suite);

end.
