{*******************************************************}
{                                                       }
{       Advanced SQL statement parser                   }
{       Classes for testing user reported bugs          }
{       Copyright (c) 2001 - 2004 AS Gaiasoft           }
{       Created by Gert Kello                           }
{                                                       }
{*******************************************************}

unit gaUserReportedBugTests;

interface

uses
  gaSQLParserTestersBase, gaBasicSQLParser, gaAdvancedSQLParser, TestFrameWork,
  gaAdvancedParserTestersBase;

type
  TgaUserReportedBugTester = class (TgaStatementTesterBase)
  published
    procedure TestBug_637750;
    procedure TestBug_637762;
    procedure TestBug_734531;
    procedure TestBug_737675;
    procedure TestBug_739045;
    procedure TestBug_764606;
    procedure TestBug_776287;
    procedure TestBug_781670;
    procedure TestBug_786468;
    procedure TestBug_786616;
    procedure TestBug_822829;
    procedure TestBug_840628;
    procedure TestBug_870871;
    procedure TestBug_876126;
    procedure TestBug_899658;
    procedure TestBug_StatementFields_GetAsString;
    procedure TestBug_StatementFields_SetAsString;
    procedure TestBug_968715;
    procedure TestBug_968715_2;
    procedure TestBug_968715_3;
    procedure TestBug_968715_4;
    procedure TestBug_974189;
    procedure TestBug_983324;
    procedure TestBug_986540;
    procedure TestBug_994328;
    procedure TestBug_NoOrderDescending;
  end;

implementation

uses
  Sysutils, gaSQLTableRefParsers, gaDeleteStm, gaSelectStm,
  gaSQLFieldRefParsers;

{
*************************** TgaUserReportedBugTester ***************************
}
procedure TgaUserReportedBugTester.TestBug_637750;
begin
  ParseStatement('SELECT SSMS.ID_SSM,'#13#10 +
                 '      (SELECT P.VLR FROM PROC) AS VLR_S,'#13#10 +
                 '      SSMS.NM_PROC'#13#10 +
                 'FROM SSMS S');
  Check(Statement.AllFields.Count = 4, 'Wrong statemetn AllFields count');
  Statement.AllFields.First;
  Check(SameText(Statement.AllFields.CurrentItem.AsString, 'SSMS.ID_SSM'), 'Wrong AllFields FirstItem.AsString');
  Statement.AllFields.Next;
  Check(SameText(Statement.AllFields.CurrentItem.AsString, '(SELECT P.VLR FROM PROC) AS VLR_S'), 'Wrong AllFields SecondItem.AsString');
  Statement.AllFields.Next;
  Check(SameText(Statement.AllFields.CurrentItem.AsString, 'P.VLR '), 'Wrong AllFields ThirdItem.AsString');
  Statement.AllFields.Next;
  Check(SameText(Statement.AllFields.CurrentItem.AsString, 'SSMS.NM_PROC'#13#10), 'Wrong AllFields FourthItem.AsString');
end;

procedure TgaUserReportedBugTester.TestBug_637762;
begin
  ParseStatement('select a, b'#13#10 +
                  'from t'#13#10 +
                  'order by b');
  Check(Statement.AllFields.Count = 2, 'Wrong AllFields count');
  Statement.AllFields.First;
  Check(SameText(Statement.AllFields.CurrentItem.AsString, 'a'), 'Wrong AllFields FirstItem.AsString');
  Statement.AllFields.Next;
  Check(SameText(Statement.AllFields.CurrentItem.AsString, 'b'#13#10), 'Wrong AllFields SecondItem.AsString');
end;

procedure TgaUserReportedBugTester.TestBug_734531;
begin
  ParseStatement('select Field1, Field2, Convert(varchar(32), FieldX) as Field3 from Table1');
  Check(Statement.AllFields.Count =3, 'Wrong AllFields count');
  Statement.AllFields.First;
  Check(SameText(Statement.AllFields.CurrentItem.AsString, 'Field1'), 'Wrong AllFields FirstItem.AsString');
  Statement.AllFields.Next;
  Check(SameText(Statement.AllFields.CurrentItem.AsString, 'Field2'), 'Wrong AllFields SecondItem.AsString');
  Statement.AllFields.Next;
  Check(SameText(Statement.AllFields.CurrentItem.AsString, 'Convert(varchar(32), FieldX) as Field3'), 'Wrong AllFields ThirdItem.AsString');
end;

procedure TgaUserReportedBugTester.TestBug_737675;
begin
  ParseStatement('SELECT FIELD1 FROM GET_STP_DATA(:PARAM1, :PARAM2)');
  Check(Statement.AllFields.Count = 1, 'Wrong AllFields count');
  Statement.AllFields.First;
  Check(SameText(Statement.AllFields.CurrentItem.AsString, 'FIELD1 '), 'Wrong AllFields FirstItem.AsString');
  Check(Statement.AllTables.Count = 1, 'Wrong table count');
  Statement.AllTables.First;
  Check(TgaSQLTable(Statement.AllTables.CurrentItem).DataReference is TgaSQLStoredProcReference, 'Wrong data reference type');
  Check(TgaSQLStoredProcReference(TgaSQLTable(Statement.AllTables.CurrentItem).DataReference).StoredProcedureParameters.Count = 2, 'Wrong stored proc param count');
end;

procedure TgaUserReportedBugTester.TestBug_739045;
begin
  ParseStatement('select count(*) from archieve');
end;

procedure TgaUserReportedBugTester.TestBug_764606;
begin
  ParseStatement('SELECT DISTINCT'#13#10 +
                  '10 AS TIPO,'#13#10 +
                  'CRM.CGC,'#13#10 +
                  'CRM.INSCEST AS IE,'#13#10 +
                  'UPPER(CRM.RAZAOSOCIAL) AS NOME,'#13#10 +
                  'UPPER(EDC.CIDADE) AS MUNICIPIO,'#13#10 +
                  'UPPER(EDC.UF) AS UF,'#13#10 +
                  'CRM.FAX,'#13#10 +
                  'DATEADD(DAY, 0, ''2003/01/01'') AS DATAINI,'#13#10 +
                  'DATEADD(DAY, 0, ''2003/12/31'') AS DATAFIM,'#13#10 +
                  '''1'' AS CODCONVENIO,'#13#10 +
                  '''2'' AS CODNATUREZA,'#13#10 +
                  '''1'' AS FINALIDADE'#13#10 +
                  'FROM'#13#10 +
                  'EMPRESAS EMP,'#13#10 +
                  'CRM_CLIENTES CRM,'#13#10 +
                  'CRM_ENDERECOS EDC'#13#10 +
                  'WHERE'#13#10 +
                  '(EDC.OBJID_CLIENTE = CRM.OBJID) AND'#13#10 +
                  '(CRM.CODCLIENTE = EMP.CODCLIENTE) AND'#13#10 +
                  '(EMP.CODEMPRESA = 1)');
end;

procedure TgaUserReportedBugTester.TestBug_776287;
begin
  ParseStatement('DELETE FROM adress WHERE adrnr=''1''');
  Check(SameText((Statement as TgaDeleteSQLStatement).StatementTable.TableName, 'adress'), 'wrong StatementTable.TableName');
end;

procedure TgaUserReportedBugTester.TestBug_781670;
begin
  ParseStatement('SELECT FIELD FROM TABLE WHERE ID = 10');
  (Statement as TgaSelectSQLStatement).WhereClause.AsString := 'ID = 20';
  CheckEquals('SELECT FIELD FROM TABLE where ID = 20', Trim(Statement.CurrentSQL.AsString), 'where clause not update correctly');
end;

procedure TgaUserReportedBugTester.TestBug_786468;
begin
  ParseStatement('select'#13#10 +
                 'table_1.field_a,'#13#10 +
                 '(select field_a'#13#10 +
                 'from table_2 join table_1 on (table_2.field_b ='#13#10 +
                 'table_1.field_b))'#13#10 +
                 'from table_1');
end;

procedure TgaUserReportedBugTester.TestBug_786616;
begin
  ParseStatement('insert into MyTable values (1)');
  StartExpectingException(EgaSQLParserException);
  ParseStatement('insert into MyTable MyTest values (1)');
  StopExpectingException('Insert table did accept table alias');
end;

procedure TgaUserReportedBugTester.TestBug_822829;
begin
  with Parser.SymbolQuotes.Add do
  begin
    StartDelimitier := '[';
    EndDelimitier := ']';
    QuotedIdentifierType := qtSymbol;
  end;
  ParseStatement('select [id] from [tablename]');
  CheckEquals('select [id] from [tablename]', Trim(Statement.CurrentSQL.AsString));
end;

procedure TgaUserReportedBugTester.TestBug_840628;
begin
  ParseStatement('SELECT a1, a2, a3 FROM TABLE order by a3, a2');
  (Statement as TgaSelectSQLStatement).WhereClause.AsString := 'a3 = 44';
  CheckEquals('SELECT a1, a2, a3 FROM TABLE where a3 = 44 order by a3, a2', Trim(Statement.AsString));
  (*  ParseStatement('select'#10#13 +
                   'LPAFACTURAS.ID_FACTURA,'#10#13 +
                   'LPAFACTURAS.ID_BANCO,'#10#13 +
                   'LPAFACTURAS.ID_PROVEEDOR,'#10#13 +
                   'LPAFACTURAS.NRO_FACTURA,'#10#13 +
                   'LPAFACTURAS.FECHA,'#10#13 +
                   'LPAFACTURAS.FECHA_CONTABLE,'#10#13 +
                   'LPAFACTURAS.MENSAJE,'#10#13 +
                   'LPAFACTURAS.IMPORTE_DESCUENTO,'#10#13 +
                   'LPAFACTURAS.IMPORTE_GASTOS_RENEGOCIACION,'#10#13 +
                   'LPAFACTURAS.CTA_GASTOS_RENEGOCIACION,'#10#13 +
                   'LPAFACTURAS.IMPORTE_EXENTO,'#10#13 +
                   'LPAFACTURAS.IMPORTE_IVA_INCLUIDO,'#10#13 +
                   'LPAFACTURAS.PORCENTAJE_IRPF,'#10#13 +
                   'LPAFACTURAS.IMPORTE_IRPF,'#10#13 +
                   'LPAFACTURAS.TIPO_FACTURA,'#10#13 +
                   'LPAFACTURAS.TIPO_CONTABILIDAD,'#10#13 +
                   'LPAFACTURAS.PORCENTAJE_IVA_INCLUIDO,'#10#13 +
                   'LPAFACTURAS.ESTA_ACTIVA,'#10#13 +
                   'LPAPROVEEDORES.RAZON_SOCIAL'#10#13 +
                   'from LPAFACTURAS join LPAPROVEEDORES on'#10#13 +
                   '(LPAFACTURAS.ID_PROVEEDOR ='#10#13 +
                   'LPAPROVEEDORES.ID_PROVEEDOR)'#10#13 +
                   'ORDER BY RAZON_SOCIAL');
    (Statement as TgaSelectSQLStatement).WhereClause.AsString := 'mayusculas (RAZON_SOCIAL) = like mayusculas(''%pro%'')';
    CheckEquals('ORDER BY RAZON_SOCIAL', Trim(Statement.AsString));*)
end;

procedure TgaUserReportedBugTester.TestBug_870871;
begin
  ParseStatement('SELECT field1, field2, field3 FROM table GROUP BY 1,2,3 ORDER BY 1,2,3');
  ParseStatement('SELECT year(field1),month(field2),sum(field3) FROM table GROUP BY year(field1),month(field2) ORDER BY 1,2');
end;

procedure TgaUserReportedBugTester.TestBug_876126;
begin
  ParseStatement('/* CORREGGE UT INCONGRUENTI SULLA PP215 */'#13#10 +
                 'UPDATE NAVDAT.SCARICOA'#13#10 +
                 ' SET   CODUT8 = ''401'''#13#10 +
                 ' WHERE CODUT8 = ''P01'' AND'#13#10 +
                 '    POLIZZA = ''6151612''');
end;

procedure TgaUserReportedBugTester.TestBug_899658;
begin
  ParseStatement('select Client, Town from Orders group by Client, Town');
  (Statement as TgaSelectSQLStatement).WhereClause.AsString := 'A = "B"';
  CheckEquals('select Client, Town from Orders where A = "B" group by Client, Town', Trim(Statement.AsString));
end;

procedure TgaUserReportedBugTester.TestBug_StatementFields_GetAsString;
begin
  ParseStatement('SELECT a1, a2, a3 FROM TABLE');
  CheckEquals('a1, a2, a3', Trim((Statement as TgaSelectSQLStatement).StatementFields.AsString), 'StatementFields.AsString incorrect');
end;

procedure TgaUserReportedBugTester.TestBug_StatementFields_SetAsString;
begin
  ParseStatement('SELECT a1, a2, a3 FROM TABLE');
  CheckEquals('a1, a2, a3', Trim((Statement as TgaSelectSQLStatement).StatementFields.AsString), 'StatementFields.AsString incorrect');
  (Statement as TgaSelectSQLStatement).StatementFields.AsString := 'b13, c33, g44';
  CheckEquals('SELECT b13, c33, g44 FROM TABLE', Trim(Statement.AsString));
end;

procedure TgaUserReportedBugTester.TestBug_968715;
begin
  ParseStatement('SELECT * FROM TABLE');
  (Statement as TgaSelectSQLStatement).OrderByClause.AsString := 'Order by field1, field2';
  CheckEquals('field1, field2', Trim((Statement as TgaSelectSQLStatement).OrderByClause.AsString));
end;

procedure TgaUserReportedBugTester.TestBug_968715_2;
begin
  ParseStatement('SELECT * FROM TABLE');
  (Statement as TgaSelectSQLStatement).GroupByClause.AsString := 'group by field1, field2';
  CheckEquals('field1, field2', Trim((Statement as TgaSelectSQLStatement).GroupByClause.AsString));
end;

procedure TgaUserReportedBugTester.TestBug_968715_3;
const
  ExpectedStatementString = 'SELECT * FROM TABLE Order by field1, field2';
begin
  ParseStatement('SELECT * FROM TABLE order by fieldX');
  (Statement as TgaSelectSQLStatement).OrderByClause.AsString := 'Order by field1, field2';
  CheckEquals(ExpectedStatementString, Trim(Statement.AsString));
end;

procedure TgaUserReportedBugTester.TestBug_968715_4;
const
  ExpectedStatementString = 'SELECT * FROM TABLE group by field1, field2';
begin
  ParseStatement('SELECT * FROM TABLE group by FieldX');
  (Statement as TgaSelectSQLStatement).GroupByClause.AsString := 'group by field1, field2';
  CheckEquals(ExpectedStatementString, Trim(Statement.AsString));
end;

procedure TgaUserReportedBugTester.TestBug_974189;
begin
  ParseStatement('select *'#13#10'from emp e, bonus b'#13#10'where e.EMPNO =b.');
end;

procedure TgaUserReportedBugTester.TestBug_983324;
begin
  ParseStatement('SELECT DISTINCT adrnr FROM adress');
  (Statement as TgaSelectSQLStatement).StatementFields.AsString := 'adrnr,id';
  CheckEquals('SELECT DISTINCT adrnr,id FROM adress', Trim(Statement.AsString));
end;

procedure TgaUserReportedBugTester.TestBug_986540;
var
  SelectStm: TgaSelectSQLStatement;
  orderItem: TgaSQLOrderByReference;
begin
  ParseStatement('SELECT * FROM adress order by field1 DESC');
  SelectStm := Statement as TgaSelectSQLStatement;
  SelectStm.OrderByClause.First;
  orderItem := SelectStm.OrderByClause.CurrentItem as TgaSQLOrderByReference;
  Check(not orderItem.OrderedAscending, 'Should be descending order');
  CheckEquals('DESC', orderItem.OrderDescriptorStr);
  ParseStatement('SELECT * FROM adress order by field1 ASC');
  SelectStm := Statement as TgaSelectSQLStatement;
  SelectStm.OrderByClause.First;
  orderItem := SelectStm.OrderByClause.CurrentItem as TgaSQLOrderByReference;
  Check(orderItem.OrderedAscending, 'Should be ascending order');
  CheckEquals('ASC', orderItem.OrderDescriptorStr);
end;

procedure TgaUserReportedBugTester.TestBug_994328;
var
  SelectStm: TgaSelectSQLStatement;
begin
  ParseStatement('select * from testTable where tableField=''T''');
  SelectStm := Statement as TgaSelectSQLStatement;
  SelectStm.WhereClause.AsString := '';
  CheckEquals('', Trim(SelectStm.WhereClause.AsString));
  CheckEquals('select * from testTable', Trim(SelectStm.AsString));
end;

procedure TgaUserReportedBugTester.TestBug_NoOrderDescending;
var
  SelectStm: TgaSelectSQLStatement;
begin
  ParseStatement('select * from testTable order by test desc');
  SelectStm := Statement as TgaSelectSQLStatement;
  SelectStm.OrderByClause.First;
  Check(not SelectStm.OrderByClause.CurrentItem.OrderedAscending);
  CheckEquals('desc', SelectStm.OrderByClause.CurrentItem.OrderDescriptorStr);
end;

end.
